import zipfile
import io
import os


def extract_ignoring_folders(zip_data):
    files = []
    with zipfile.ZipFile(io.BytesIO(zip_data)) as z:
        for filename in z.namelist():
            if os.path.exists(filename.split("/")[-1]):
                os.remove(filename.split("/")[-1])
            z.extract(filename)
            os.rename(filename, filename.split("/")[-1])
            os.removedirs("/".join(filename.split("/")[:-1]))
            files.append(filename.split("/")[-1])
    return files



import requests
import platform

from ..archive.zip import extract_ignoring_folders
from ..imports import import_rel

class BrowserClient:
    driver = None
    selenium = None

    def __init__(self):
        raise NotImplementedError


class ChromeClient(BrowserClient):
    def __init__(self, selenium):
        self.selenium = selenium
        self.download_chromedriver()
        options = self.selenium.webdriver.chrome.options.Options()
        options.add_argument("--headless")
        options.add_argument("--disable-gpu")
        options.add_argument("--no-sandbox")
        options.add_argument("--disable-dev-shm-usage")
        self.driver = self.selenium.webdriver.Chrome(options=options)

    def download_chromedriver(self):
        chrome_version_data = requests.get(
            "https://googlechromelabs.github.io/chrome-for-testing/last-known-good-versions-with-downloads.json",
            timeout=30
        ).json()
        stable_chromedriver_downloads = chrome_version_data["channels"]["Stable"][
            "downloads"
        ]["chromedriver"]

        system = platform.system()
        if system == "Linux":
            found_platform = "linux64"
        elif system == "Darwin":
            found_platform = "mac-x64"
        elif system == "Windows":
            if "64bit" in platform.architecture():
                found_platform = "win64"
            else:
                found_platform = "win32"
        else:
            raise NotImplementedError(f"Unsupported platform {system}")

        for possible_donwload in stable_chromedriver_downloads:
            if possible_donwload["platform"] == found_platform:
                chromedriver_download = possible_donwload
                break
        else:
            raise FileNotFoundError(
                f"Could not find chromedriver for {found_platform}")

        chromedriver_zip = requests.get(
            chromedriver_download["url"], timeout=30)

        return extract_ignoring_folders(chromedriver_zip.content)


def get_selenium():
    return import_rel("selenium.webdriver")



from typing import Any, Callable
from functools import wraps


def entrypoint(func: Callable) -> Callable:
    """Execute the function to decorate function if the script is run directly.

    This is equivalent to:
    ```python
    if __name__ == "__main__":
        func()
    ```

    (where `func` is the decorated function)

    Args:
        func (Callable): The function to decorate

    Returns:
        Callable: The decorated function
    """
    if func.__module__ == "__main__":
        func()
    return func


def bind(*args, **kwargs) -> Callable:
    """Bind arguments to a function.

    Example:
    ```python
    from jjjxutils.decorators import bind

    @bind(2, 3)
    def addition(x, y) -> int:
        return x + y
    ```

    This will make `addition` a function that takes no arguments and returns 5.

    Returns:
        Callable: _description_
    """

    def decorator(func: Callable) -> Callable:
        @wraps(func)
        def wrapper(*args_, **kwargs_) -> Any:
            return func(*args, *args_, **kwargs, **kwargs_)

        return wrapper

    return decorator


def compute(func: Callable) -> Any:
    """Execute the function and save the result.

    This is equivalent to:
    ```python
    func = func()
    ```

    This is useful when you want to execute code or calculate a value at import time, but still want to keep it in a seperate scope.

    Args:
        func (Callable): The function to decorate

    Returns:
        Any: The return value of the function
    """
    return func()



import base64
import json

from ..web.browser_client import BrowserClient


class HtmlRenderer:
    def __init__(self, browser_client: BrowserClient):
        self.browser_client = browser_client
        self.driver = browser_client.driver

    def save_current_pdf(self, dest, landscape=False):
        resource = "/session/%s/chromium/send_command_and_get_result" % self.driver.session_id
        url = self.driver.command_executor._url + resource
        body = json.dumps(
            {
                "cmd": "Page.printToPDF",
                "params": {
                    "landscape": landscape,
                    "displayHeaderFooter": False,
                    "printBackground": True,
                    "preferCSSPageSize": False,
                },
            }
        )
        response = self.driver.command_executor._request("POST", url, body)
        with open(dest, "wb") as file:
            file.write(base64.b64decode(response.get("value")["data"]))

    def save_current_png(self, dest, landscape=False):
        if landscape:
            self.driver.set_window_size(1414, 1000)
        else:
            self.driver.set_window_size(1000, 1414)
        self.driver.save_screenshot(dest)

    def save_as_pdf(self, uri, dest, landscape=False):
        self.driver.get(uri)
        self.save_current_pdf(dest, landscape)

    def save_as_png(self, uri, dest, landscape=False):
        self.driver.get(uri)
        self.save_current_png(dest, landscape)
